{-# LANGUAGE ExistentialQuantification #-}
{-# OPTIONS -Wall #-}
module StrategyManager where

import System.Random

import Datatypes
import Datatypes.Game

type GameComputation = [StrategyWrapper]

data StrategyWrapper = forall a . Strategy a => MkStrategyWrapper a
                     | FinishedGame ([Command], Int)


data NullStrategy1 = NullS1



initWrapper :: Strategy a => a -> StrategyWrapper
initWrapper = MkStrategyWrapper

class Strategy a where
  initst  :: Game -> StdGen -> Maybe [Command] -> a
  advance :: a -> Either a ([Command], Int) 
  getbest :: a -> ([Command], Int)


advanceWrapper :: StrategyWrapper -> StrategyWrapper 
advanceWrapper (FinishedGame result)  = (FinishedGame result)
advanceWrapper (MkStrategyWrapper st) = wrapResult $ advance st 
  where
    wrapResult (Left nst) = MkStrategyWrapper nst
    wrapResult (Right result) = FinishedGame result 

finishedWrapper :: StrategyWrapper -> Bool
finishedWrapper (FinishedGame _) = True
finishedWrapper _ = False

getbestWrapper ::  StrategyWrapper -> ([Command], Int) 
getbestWrapper (FinishedGame result)  = result
getbestWrapper (MkStrategyWrapper st) = getbest st

--- puo essere parallelizzato ---
finishedGameComputation :: GameComputation -> Bool
finishedGameComputation gc = and $ map finishedWrapper gc

-- Ritorna i comandi con i punti piu l indice della strategia
getBestGameComputation :: GameComputation ->  (([Command], Int),Int)
getBestGameComputation gc = foldl bestgame (([], 0), 0) (map getbestWrapper gc)
  where
    bestgame (b,i) a = if (snd a) > (snd b)
                       then (a,i+1)
                       else (b,i)

advanceGameComputation :: GameComputation -> GameComputation 
advanceGameComputation gc = map advanceWrapper gc

instance Strategy NullStrategy1 where
  initst _ _ _ = NullS1
  advance  _ = Left NullS1
  getbest  _ = ([],0)


data NullStrategy2 = NullS2

instance Strategy NullStrategy2 where
  initst _ _ _ = NullS2
  advance  _ = Left NullS2
  getbest  _ = ([],0)

